@@ -44,5 +44,5 @@ def create_model(config_path, device):
     config = OmegaConf.load(config_path)
     config["model"]["params"]["cond_stage_config"]["params"] = {"device": device}
     model = instantiate_from_config(config.model).cpu()
-    print(f"Loaded model config from [{config_path}]")
+    # print(f"Loaded model config from [{config_path}]")
     return model
