@@ -92,7 +92,7 @@ class DDIMSampler(object):
         )
 
     @torch.no_grad()
-    async def sample(
+    def sample(
         self,
         S,
         batch_size,
@@ -146,9 +146,9 @@ class DDIMSampler(object):
         # sampling
         C, H, W = shape
         size = (batch_size, C, H, W)
-        print(f"Data shape for DDIM sampling is {size}, eta {eta}")
+        # print(f"Data shape for DDIM sampling is {size}, eta {eta}")
 
-        samples = await self.ddim_sampling(
+        samples = self.ddim_sampling(
             conditioning,
             size,
             callback=callback,
@@ -171,7 +171,7 @@ class DDIMSampler(object):
         return samples
 
     @torch.no_grad()
-    async def ddim_sampling(
+    def ddim_sampling(
         self,
         cond,
         shape,
@@ -196,7 +196,7 @@ class DDIMSampler(object):
         device = self.model.betas.device
         b = shape[0]
         if x_T is None:
-            img = torch.randn(shape, device=device, dtype=cond['c_concat'][0].dtype)
+            img = torch.randn(shape, device=device, dtype=cond["c_concat"][0].dtype)
         else:
             img = x_T
 
@@ -222,7 +222,7 @@ class DDIMSampler(object):
             else np.flip(timesteps)
         )
         total_steps = timesteps if ddim_use_original_steps else timesteps.shape[0]
-        print(f"Running DDIM Sampling with {total_steps} timesteps")
+        # print(f"Running DDIM Sampling with {total_steps} timesteps")
 
         iterator = tqdm(time_range, desc="DDIM Sampler", total=total_steps)
 
@@ -258,9 +258,10 @@ class DDIMSampler(object):
             )
             img, pred_x0 = outs
             if callback:
-                await callback(i)
+                callback(i)
             if img_callback:
-                await img_callback(pred_x0, i)
+                # (step: int, timestep: int, latents: torch.FloatTensor)
+                img_callback(i, ts, pred_x0)
 
         return img
 
@@ -342,8 +343,12 @@ class DDIMSampler(object):
         )
         # select parameters corresponding to the currently considered timestep
         a_t = torch.full((b, 1, 1, 1), alphas[index], device=device, dtype=torch_dtype)
-        a_prev = torch.full((b, 1, 1, 1), alphas_prev[index], device=device, dtype=torch_dtype)
-        sigma_t = torch.full((b, 1, 1, 1), sigmas[index], device=device, dtype=torch_dtype)
+        a_prev = torch.full(
+            (b, 1, 1, 1), alphas_prev[index], device=device, dtype=torch_dtype
+        )
+        sigma_t = torch.full(
+            (b, 1, 1, 1), sigmas[index], device=device, dtype=torch_dtype
+        )
         sqrt_one_minus_at = torch.full(
             (b, 1, 1, 1), sqrt_one_minus_alphas[index], device=device, dtype=torch_dtype
         )
