@@ -848,7 +848,7 @@ class UniPC:
                 )
         return x_t, model_t
 
-    async def sample(
+    def sample(
         self,
         x,
         steps=20,
@@ -882,7 +882,8 @@ class UniPC:
                 model_prev_list = [self.model_fn(x, vec_t)]
 
                 iterator.update(1)
-                if img_callback: await img_callback(x, 0)
+                if img_callback:
+                    img_callback(0, 0, x)
 
                 t_prev_list = [vec_t]
                 # Init the first `order` values by lower order multistep DPM-Solver.
@@ -904,7 +905,8 @@ class UniPC:
                     t_prev_list.append(vec_t)
 
                     iterator.update(1)
-                    if img_callback: await img_callback(x, init_order)
+                    if img_callback:
+                        img_callback(init_order, 0, x)
 
                 for step in range(order, steps + 1):
                     vec_t = timesteps[step].expand(x.shape[0])
@@ -936,7 +938,8 @@ class UniPC:
                             model_x = self.model_fn(x, vec_t)
                         model_prev_list[-1] = model_x
                     iterator.update(1)
-                    if img_callback: await img_callback(x, step)
+                    if img_callback:
+                        img_callback(step, 0, x)
         else:
             raise NotImplementedError()
         if denoise_to_zero:
