@@ -4,11 +4,9 @@ import os
 
 from starlette.staticfiles import StaticFiles
 
-os.environ["PYTORCH_ENABLE_MPS_FALLBACK"] = "1"
 import threading
 import time
 from pathlib import Path
-from urllib.parse import urlparse
 
 import torch
 import uvicorn
@@ -17,8 +15,7 @@ from fastapi import FastAPI, File, Form
 from fastapi.middleware.cors import CORSMiddleware
 from loguru import logger
 from starlette.responses import FileResponse, StreamingResponse
-from torch.hub import download_url_to_file
-from typer import Typer, Option
+from typer import Option
 
 from tldream.socket_manager import SocketManager
 from tldream.util import process, load_img, torch_gc, pil_to_bytes, init_pipe, get_ip
@@ -39,11 +36,6 @@ web_static_folder = os.path.join(current_dir, "out")
 app.mount("/static", StaticFiles(directory=web_static_folder), name="static")
 sio = SocketManager(app=app)
 
-typer_app = Typer(
-    add_completion=False,
-    pretty_exceptions_show_locals=False,
-)
-
 controlled_model = None
 lock = threading.Lock()
 
@@ -126,18 +118,14 @@ def run(
     return response
 
 
-@typer_app.command()
-def start(
-    listen: bool = Option(False, help="If true, start server at 0.0.0.0"),
-    port: int = Option(4242),
-    device: str = Option("cuda", help="Device to use (cuda, cpu or mps)"),
-    model: str = Option(
-        "runwayml/stable-diffusion-v1-5",
-        help="Any HuggingFace Stable Diffusion model id",
-    ),
-    low_vram: bool = Option(False, help="Use low vram mode"),
-    fp32: bool = Option(False, help="Use float32 mode"),
-    nsfw_filter: bool = Option(True),
+def main(
+    listen: bool,
+    port: int,
+    device: str,
+    model: str,
+    low_vram: bool,
+    fp32: bool,
+    nsfw_filter,
 ):
     from diffusers.utils import DIFFUSERS_CACHE
 
@@ -163,7 +151,3 @@ def start(
     else:
         host = "127.0.0.1"
     uvicorn.run(app, host=host, port=port)
-
-
-def main():
-    typer_app()
